"""
OpenStreetMap Data Primitives
Released into the public domain
2010 Andreas Stricker <andy@knitter.ch>
"""

class Container(object):
    OSM_PRIMITIVES = {}
    def __init__(self):
        self._primitives = {}
        for v in self.OSM_PRIMITIVES.itervalues():
            self._primitives[v] = {}
        self._bounds = []
        self.version = '0.6'
        self.generator = 'osm.py'
    def getStats(self):
        r = {
            'version': self.version,
            'generator': self.generator,
        }
        for k, v in self.OSM_PRIMITIVES.iteritems():
            r[k] = len(self._primitives[v])
        return r
    def getNodes(self):
        return self._primitives[Node]
    def getWays(self):
        return self._primitives[Way]
    def getRelations(self):
        return self._primitives[Relation]
    def getBounds(self):
        return self._bounds
    def find(self, condition, primitive=None):
        results = []
        if primitive is None:
            for v in self.OSM_PRIMITIVES.itervalues():
                for item in self._primitives[v].itervalues():
                    if condition(item):
                        results.append(item)
        else:
            if isinstance(primitive, type):
                k = primitive
            else:
                k = self.OSM_PRIMITIVES[primitive]
            for item in self._primitives[k].itervalues():
                if condition(item):
                    results.append(item)
        return results
    def addBound(self, bound):
        self._bounds.append(bound)
    def add(self, item):
        if not item.isValid():
            raise Exception("Item is not valid")
        self._primitives[type(item)][item.getAttribute('id')] = item
    def get(self, key, osmType=None):
        if osmType is None:
            for v in self.OSM_PRIMITIVES.itervalues():
                try:
                    return self._primitives[v][key]
                except KeyError:
                    pass
            else:
                raise KeyError(repr(key))
        else:
            e = self._primitives[key]
            if not isinstance(osmType, type):
                osmType = self.OSM_PRIMITIVES[osmType]
            if not isinstance(e, osmType):
                raise Exception("Id '%s' is not of type %s: %s" % (key, osmType, repr(e)))
            return e

class Bounds(object):
    def __init__(self, **kwargs):
        self.setBounds(**kwargs)
    def setBounds(self, minlat=None, maxlat=None, minlon=None, maxlon=None):
        self._minlat = minlat
        self._maxlat = maxlat
        self._minlon = minlon
        self._maxlon = maxlon
    def setBox(self, box):
        self.minlat, self.minlon, self.maxlat, self.maxlon = tuple([
            float(x) for x in box.split(',')
        ])
    def setOrigin(self, origin):
        self.origin = origin
       
class OsmPrimitive(object):
    def __init__(self):
        self._valid = False
        self._attributes = {}
        self._tags = {}
    def __repr__(self):
        if ('id' in self._attributes) and (self._attributes['id'] is not None):
            return '<%s id=%s>' % (type(self).__name__, self._attributes['id'])
        else:
            return '<%s object at %x>' % (type(self).__name__, id(self))
    def isValid(self):
        return self._valid
    def getAttribute(self, k):
        if not k in self.ATTRS:
            raise Exception("Key '%s' is not known" % k)
        return self._attributes.get(k, None)
    def setAttribute(self, k, v):
        if not k in self.ATTRS:
            raise Exception("Key '%s' is not known" % k)
        if k == 'id':
            self._valid = True
        self._attributes[k] = v
    def getAttributes(self):
        return self._attributes
    attributes = property(getAttributes)
    def addTag(self, k, v):
        self._tags[k] = v
    def getTag(self, k):
        return self._tags[k]
    def getTags(self):
        return self._tags
    tags = property(getTags)
    def iterTags(self):
        return self._tags.iteritems()
    def iterAttrs(self):
        for k in self.ATTRS:
            yield k

class UnresolvedReference(OsmPrimitive):
    NAME = 'ref'
    ATTRS = ('id')
    def __init__(self):
        OsmPrimitive.__init__(self)
        self._typeHint = None
    def getRef(self):
        return self.getAttribute('id')
    def setRef(self, ref, typeHint=None):
        self._typeHint = typeHint
        self._attributes['id'] = ref
    def getTypeHint(self):
        return self._typeHint

class Node(OsmPrimitive):
    NAME = 'node'
    ATTRS = ('id', 'uid', 'user', 'timestamp', 'version', 'changeset', 'lat', 'lon', 'visible')
    def __init__(self):
        OsmPrimitive.__init__(self)
Container.OSM_PRIMITIVES['node'] = Node

class Way(OsmPrimitive):
    NAME = 'way'
    ATTRS = ('id', 'uid', 'user', 'timestamp', 'version', 'changeset', 'visible')
    def __init__(self):
        OsmPrimitive.__init__(self)
        self._nodes = []
    def addNode(self, n):
        self._nodes.append(n)
    def getNodes(self):
        return self._nodes
    def iterNodes(self):
        for n in self._nodes:
            yield n
Container.OSM_PRIMITIVES['way'] = Way

class Relation(OsmPrimitive):
    NAME = 'relation'
    ATTRS = ('id', 'uid', 'user', 'timestamp', 'version', 'changeset', 'visible')
    class Member(object):
        def __init__(self, role, element):
            self._role = role
            self._element = element
        def isNode(self):
            return isinstance(self._element, Node)
        def isWay(self):
            return isinstance(self._element, Way)
        def isRelation(self):
            return isinstance(self._element, Relation)
            
    def __init__(self):
        OsmPrimitive.__init__(self)
        self._members = []

    def addMember(self, role, member):
        self._members.append(Relation.Member(role, member))
    def getMembers(self):
        return self._members
    def iterMembers(self):
        for m in self._members:
            yield m
Container.OSM_PRIMITIVES['relation'] = Relation

