"""
OpenStreetMap Data OSM xml dialect parser
Released into the public domain
2010 Andreas Stricker <andy@knitter.ch>
"""
import xml.parsers.expat
import osm
try:
    import bz2
except ImportError: pass
try:
    import gzip
except ImportError: pass

class OsmReader(object):
    def __init__(self):
        self.container = None

    def _init_parser(self):
        self.parser = xml.parsers.expat.ParserCreate()
        self.parser.StartElementHandler = self._start_element
        self.parser.EndElementHandler = self._end_element
        self.container = osm.Container()

    def parse(self, filename):
        self._init_parser()
        if filename.endswith('.bz2'):
            f = bz2.BZ2File(filename, 'r')
        elif filename.endswith('.gz'):
            f = gzip.GzipFile(filename, 'r')
        else:
            f = file(filename, 'r')
        self._cur = None
        try:
            self.parser.ParseFile(f)
        finally:
            f.close()
        return self.container

    def _start_element(self, name, attrs):
        #print 'Start element:', name, attrs
        try:
            getattr(self, "_parse_" + name)(attrs)
        except AttributeError:
            raise Exception("Unknown element '%s'" % name)
        
    def _end_element(self, name):
        if name in ['node', 'way', 'relation']:
            self._cur = None

    def _parse_osm(self, attrs):
        self.container.version = attrs['version']
        self.container.generator = attrs.get('generator', '')

    def _parse_bounds(self, attrs):
        minlat = float(attrs['minlat'])
        maxlat = float(attrs['maxlat'])
        minlon = float(attrs['minlon'])
        maxlon = float(attrs['maxlon'])
        self.container.addBound(osm.Bounds(
            minlat=minlat, 
            minlon=minlon,
            maxlat=maxlat,
            maxlon=maxlon
        ))
    def _parse_bound(self, attrs):
        b = osm.Bounds()
        b.setBox(attrs['box'])
        b.setOrigin(attrs['origin'])
        self.container.addBound(b)
    def _parse_tag(self, attrs):
        self._cur.addTag(attrs['k'], attrs['v'])
    def _parse_node(self, attrs):
        self._cur = osm.Node()
        for k in self._cur.iterAttrs():
            if k in attrs:
                self._cur.setAttribute(k, attrs[k])
        self.container.add(self._cur)
    def _parse_tag(self, attrs):
        self._cur.addTag(attrs['k'], attrs['v'])
    def _parse_way(self, attrs):
        self._cur = osm.Way()
        for k in self._cur.iterAttrs():
            if k in attrs:
                self._cur.setAttribute(k, attrs[k])
        self.container.add(self._cur)
    def _parse_nd(self, attrs):
        ref = attrs['ref']
        nd = self.container.get(ref)
        self._cur.addNode(nd)
    def _parse_relation(self, attrs):
        self._cur = osm.Relation()
        for k in self._cur.iterAttrs():
            if k in attrs:
                self._cur.setAttribute(k, attrs[k])
        self.container.add(self._cur)
    def _parse_member(self, attrs):
        role = attrs['role']
        ref = attrs['ref']
        mtype = attrs['type']
        try:
            member = self.container.get(ref, mtype)
        except KeyError:
            # relation must not necessary be complete in osm
            member = osm.UnresolvedReference()
            member.setRef(ref, mtype)
        self._cur.addMember(role, member)

def main():
    """ Test """
    import sys
    r = OsmReader()
    c = r.parse(sys.argv[1])
    print c.getStats()

if __name__ == '__main__':
    main()
